"use strict"; 
this.name           = "smivs-generic-ship-script.js"; 
this.author         = "Smivs (additional code by Capt Murphy)"; 
this.copyright      = "This script is hereby placed in the public domain."; 
this.version        = "4.1.3"; 
this.description    = "Script to add visible damage when ship throws sparks, and render ship derelict on ejection of pilot." 

this.shipEnergyIsLow = function() // switches diffuse_map to -damaged version.
{
    if(this.ship.isDerelict){return;} // don't want to be switching hull lights back on if already derelict.
	this.switchToDamagedTex(); // call function to switch textures - seperate function as also called from this.shipLaunchedEscapePod
}
this.switchToDamagedTex = function()
{
	var texture = this.ship.scriptInfo.smivs_texture; // uses scriptInfo key to build texture name
	var diffuse = this.ship.scriptInfo.smivs_diffuse; // uses scriptInfo key to build diffuse map name
	var emission = this.ship.scriptInfo.smivs_emission; // uses scriptInfo key to build emission map name
	var material = new Object(); // declare variable
	material[texture] = {diffuse_map:diffuse, emission_map: emission}; // set up material entry
    this.ship.setMaterials(material); // apply material
}
this.shipLaunchedEscapePod = function()
{
  if(this.ship.isValid)
    {
	this.switchToDamagedTex();//double check that ship is showing damaged texture with hull light emission map on.
	this.switchEngineTexTimer = new Timer(this, this.switchEngineTex,1); // timer to switch engines textures to derelict version after 1 second
	this.rotateDerelictTimer = new Timer(this, this.rotateDerelict,3); // timer to start derelict rotation after 3 seconds
	this.switchHullTexTimer = new Timer(this, this.switchHullTex,5); // timer to switch off hull lights emission map after 5 seconds
	this.switchFlashersAITimer = new Timer(this, this.switchFlashersOff,7); // timer to switch off flashers after 7 seconds
    }
}
this.switchEngineTex = function()
{
    var texture; // declare variable
	var diffuse; //declare variable
	var material = new Object(); // declare variable
	var subents = this.ship.subEntities; // create array of subents
	var counter = 0; // set up loop to check if subent is an engine and if so change texture to derelict version
	var subentsLength = subents.length;
    for (counter = 0; counter < subentsLength;counter++)
    {
	 if (subents[counter].scriptInfo.smivs_texture)
	 {
	    texture = subents[counter].scriptInfo.smivs_texture; // uses scriptInfo key to build texture name
		diffuse = subents[counter].scriptInfo.smivs_diffuse; // uses scriptInfo key to build diffuse map name
		material[texture] = {diffuse_map:diffuse}; // set up material entry
		subents[counter].setMaterials(material); // sets material
	 }
	}
}
this.rotateDerelict = function()
{
 this.rotateVector = Vector3D.randomDirection(); // creates a random axis of rotation.
 this.rotateSpeedFactor = (Math.random() * (0.05) + 0.05); // creates a random rotate speed factor between 0.05 and 0.1 - 0.1 will result in a rotation speed of approx 1 full rotation every 63 seconds, 0.05 in a rotation speed of approx 1 full rotation every 126 seconds.
 this.rotateDerelictCallBack = addFrameCallback(this.rotateFunction.bind(this)); // create FrameCallback
}
this.rotateFunction = function(delta) // delta is the time in game seconds since the last frame.
{
 if (!this.ship.isValid){removeFrameCallback(this.rotateDerelictCallback);delete this.rotateDerelictCallback;return;} // end FrameCallback is ship has died;
 if (delta === 0){return;} // do nothing if game is paused;
 var newOrientation = this.ship.orientation.rotate(this.rotateVector,delta*this.rotateSpeedFactor); // calculates new orientation, using delta as a factor so that rotation speed is constant across varying frame rates. delta*this.rotateSpeedFactor is a value in radians. There are approx 6.3 radians in a full rotation.
 this.ship.orientation = newOrientation; // applies new orientation;
}
this.switchHullTex = function()
{
	var texture = this.ship.scriptInfo.smivs_texture; // uses scriptInfo key to build texture name
	var diffuse = this.ship.scriptInfo.smivs_diffuse; // uses scriptInfo key to build diffuse map name
	var material = new Object(); // declare variable
	material[texture] = {diffuse_map:diffuse}; // set up material entry
    this.ship.setMaterials(material); // apply material
}

this.switchFlashersOff = function()
{
	this.ship.lightsActive = false; // turns flashers off
}

this.shipDied = function ()
{
        var deathblast = parseInt(this.ship.scriptInfo.smivs_deathblast); // uses scriptInfo key for number of deathblasts
		var extraAlloys = parseInt(this.ship.scriptInfo.smivs_extraAlloys); // uses scriptInfo key for max number extra alloys
		var extraCargo = parseInt(this.ship.scriptInfo.smivs_extraCargo); // uses scriptInfo key for max number extra cargo
        if (deathblast > 0){system.addShips("smivs-deathblast1", deathblast, this.ship.position, 2)};
        if (extraAlloys > 0){system.addShips("alloy", Math.ceil(Math.random() * extraAlloys), this.ship.position, 25)};
        if (extraCargo > 0){system.addShips("smivs-toughened-barrel", Math.ceil(Math.random() * extraCargo), this.ship.position, 25)};
}





